tictactoe <- function() {
  
  ### _______________________________________________________________###  
  
  ### This function is a simple two-player version of the game TicTacToe
  ### Written by Noah Meier
  ### Date: 2021, October 21
  
  ### _______________________________________________________________###  
  
  ### Read me!
  ### The players are allowed to choose in turns a field within the 3x3 grid.
  ### The first one who manages to get 3 fields in a line (vertical, horizontal or diagonal) wins!
  ### You choose a field by selecting first the column and then the row (just follow the instructions of the function).
  ### If all the fields are filled without any player having a line with 3 fields, the game is drawn.
  ### If you want to stop the R function before the game is finished you can press "esc".
  
  ### _______________________________________________________________###  
  
  ### I have been looking for a nice way to plot a table and have found a script on github, see link below.
  ### The ugly thing is, if I load this script hundreds of different variables and functions are getting loaded into the environment.
  ### In order to avoid this, I extracted the functions needed from this script and source these locally -> PlotTable.txt.
  ### I copied the header of the script into my extract, in order to provide a reference for the author of the script.
  ### source(gzcon(url('https://github.com/systematicinvestor/SIT/raw/master/sit.gz', 'rb')))
  
  ### Sources a function for plotting tables
  source("Data/PlotTable.txt") #the file needs to be in the working directory
  
  ### Create a matrix, which can be used a play board
  ### Plot.table is used to plot the dataframe which is nicer than just printing it to the console.
  ### The argument smain in plot.table writes a text in the upper left corner, which is used here to call the player on the round.
  
  board <- matrix(c(rep("",9)),nrow=3,ncol=3)
  rownames(board) <- c(1:3)
  colnames(board) <- c("A","B","C")
  color.matrix <- matrix(nrow = 3, ncol = 3)
  color.matrix[1:9] <- "white"
  plot.table(board, highlight = color.matrix, smain = "-Player X-")
  
  ### i counts the number of fields that were filled and stops the game when all the 9 slots ar filled.
  
  i <- 0
  while (i<9) {
    
    ### _______________________________________________________________###
    ### Player "X" 
    ### _______________________________________________________________###
    
    ### Player "X" selects the next field
    ### There are while loops after each readline command checking whether the entries are valid.
    ### An outer while loop checks whether the chosen field is still free.
    ###! There is probably a more elegant and faster way to code this though ...
    
    print("Player X it is your turn!")
    choose_c <- readline("Select the column of your next cross (A/B/C).")
    while (!(choose_c %in% c("A","B","C"))) {
      choose_c <- readline("That's not a valid column index, choose again! (A/B/C).")}
    
    choose_r <- readline("Select the row for your next cross (1/2/3).")
    while (!(choose_r %in% c(1:3))) {
      choose_r <- readline("That's not a valid row index, choose again! (1/2/3).")}
    
    while (board[choose_r, choose_c] != "") {
      print("This is not a free spot! Choose again.")
      
      choose_c <- readline("Select the column of your next cross (A/B/C).")
      while (!(choose_c %in% c("A","B","C"))) {
        choose_c <- readline("That's not a valid column index, choose again! (A/B/C).")}
      
      choose_r <- readline("Select the row for your next cross (1/2/3).")
      while (!(choose_r %in% c(1:3))) {
        choose_r <- readline("That's not a valid row index, choose again! (1/2/3).")}
    }
    
    board[choose_r, choose_c] <- "X"
    i <- i+1
    
    ### _______________________________________________________________###
    
    ### Check through all possible winning situations, whether player "O" has three O in a line.
    ### Just writing out all possible situations is not very nice coding, but as there are only 8 possible winning situations, it is feasible.
    
    if(all(board[c(1:3),1] == "X") || all(board[c(1:3),2] == "X") || all(board[c(1:3),3] == "X") || all(board[1,c(1:3)]  == "X") || all(board[2,c(1:3)]  == "X") || all(board[3,c(1:3)]  == "X") || all(c(board[1,1],board[2,2],board[3,3])  == "X") || all(c(board[3,1],board[2,2],board[1,3]) == "X")) 
    {
      plot.table(board, highlight = color.matrix, smain = "-Player X wins!-")
      print("Congratulations Player X, you won!")
      print("End of game.")
      break}
    
    ### Check whether all fields are already filled. 
    ### Only if all fields are filled, the games stops and declares the match drawn.
    
    if (i == 9) {
      print("Oh no, there is no winner!")
      plot.table(board, highlight = color.matrix, smain = "-Draw!-")
      break
    }
      
    plot.table(board, highlight = color.matrix, smain = "-Player O-")
    

    ### _______________________________________________________________###
    ### Player "O" 
    ### _______________________________________________________________###
    
    ### Player "O" selects the next field.
    ### There are while loops after each readline command checking whether the entries are valid.
    ### An outer while loop checks whether the chosen field is still free.
    ###! There is probably a more elegant and faster way to code this though ...
    
    print("Player O it is your turn!")
    choose_c <- readline("Select the column of your next cross (A/B/C).")
    while (!(choose_c %in% c("A","B","C"))) {
      choose_c <- readline("That's not a valid column index, choose again! (A/B/C).")}
    
    choose_r <- readline("Select the row for your next cross (1/2/3).")
    while (!(choose_r %in% c(1:3))) {
      choose_r <- readline("That's not a valid row index, choose again! (1/2/3).")}
    
    while (board[choose_r, choose_c] != "") {
      print("This is not a free spot! Choose again.")
      
      choose_c <- readline("Select the column of your next cross (A/B/C).")
      while (!(choose_c %in% c("A","B","C"))) {
        choose_c <- readline("That's not a valid column index, choose again! (A/B/C).")}
      
      choose_r <- readline("Select the row for your next cross (1/2/3).")
      while (!(choose_r %in% c(1:3))) {
        choose_r <- readline("That's not a valid row index, choose again! (1/2/3).")}
      
    }  
    
    board[choose_r, choose_c] <- "O"
    i <- i+1
    
    ### Check through all possible winning situations, whether player "O" has three O in a line.
    ### Just writing out all possible situations is not very nice coding, but as there are only 8 possible winning situations, it is feasible.    
    
    if(all(board[c(1:3),1] == "O") || all(board[c(1:3),2] == "O") || all(board[c(1:3),3] == "O") || all(board[1,c(1:3)]  == "O") || all(board[2,c(1:3)]  == "O") || all(board[3,c(1:3)]  == "O") || all(c(board[1,1],board[2,2],board[3,3])  == "O") || all(c(board[3,1],board[2,2],board[1,3]) == "O")) 
      
    {      
      plot.table(board, highlight = color.matrix, smain = "-Player O wins!-")
      print("Congratulations Player O, you won!")
      print("End of game.")
      break
    }
    
    ### Check whether all fields are already filled. 
    ### Only if all fields are filled, the games stops and declares the match drawn.
    
    if (i == 9) {
      print("Oh no, there is no winner!")
      plot.table(board, highlight = color.matrix, smain = "-Draw!-")
      break
    }  
    
    plot.table(board, highlight = color.matrix, smain = "-Player X-")
      
      
    }
  }


